@@ -1,61 +1,46 @@
 # THIS IS THE ENTIRE "RADIO" FOR QUISK RUNNING AS A CONTROL HEAD
 # No real radio hardware is attached to the control_head computer.
 #
-# This software is Copyright (C) 2021 by Ben Cahill and 2006-2021 by James C. Ahlstrom.,
+# This software is Copyright (C) 2021-2022 by Ben Cahill and 2006-2022 by James C. Ahlstrom.,
 # and is licensed for use under the GNU General Public License (GPL).
 # See http://www.opensource.org.
 # Note that there is NO WARRANTY AT ALL.  USE AT YOUR OWN RISK!!
 #
-# This file, quisk_hardware_control_head.py, allows a radio-less (control_head) Quisk,
-# running on this computer, to ccnnect to a remote (remote_radio) instance of Quisk
+# This file, control_common.py, along with a radio-specific file, e.g.
+# control_softrock.py, control_hermes.py, or similar, allows a radio-less (control_head) Quisk,
+# running on this computer, to connect to a remote (remote_radio) instance of Quisk
 # that runs on a separate computer.
 #
 # The remote_radio Quisk controls an attached real radio, and uses the hardware file
-# quisk_hardware_remote_radio.py to communicate with this control_head Quisk
-# computer via network connection.
-#
-# Set up the control head Quisk by creating a new Radio, then configuring that Radio's
-# Hardware (tab) --> "Hardware File Path" to point to this file (use the "Change"
-# button to help you find it:  quisk_hardware_control_head.py).  Edit this file
-# to point to the remote_radio computer, e.g.:
-#
-#   self.remote_radio_ip = '192.168.1.60'	# Remote Quisk computer's ip
-#   self.remote_ctl_base_port = 4585		# Remote Control base port
+# remote_common.py, along with a radio-specific file, e.g. remote_softrock.py,
+# remote_hermes.py, or similar, to communicate with this control_head Quisk
+# computer via a network connection.
 #
 # The remote_radio computer should be set up with a static IP address, so that you know
 # where to point the control_head.  The control head computer may, however, use dynamic
 # addressing; the remote radio computer will read the control head address when the
 # remote control connection is made.
 #
-# You should be able to use quisk_hardware_control_head.py along with any/all means of control that
-# you normally use to control Quisk, including serial ports, MIDI, and hamlib/rigctl interfaces.
-#
-# (UNTESTED!) If you want to use quisk_hardware_hamlib.py, install that "on top of" quisk_hardware_model,
-# i.e. make sure it contains (as usual):
-#      from quisk_hardware_model import Hardware as BaseHardware
-# then edit quisk_hardware_control_head.py to install "on top of" quisk_hardware_hamlib.py, i.e.:
-#      from quisk_hardware_hamlib import Hardware as BaseHardware
-#
 # The main control interface between control_head and remote_radio is via a TCP port;
-# this uses very low bandwidth.  All functional control, including CW keyihg, is done via this port.
+# this uses very low bandwidth.  All functional control, including CW keying, is done via this port.
 #
 # There are 3 additional ports, all UDP, using low to moderate bandwidth:
 # -- Receive graph/waterfall data from the remote_radio Quisk
 # -- Receive radio sound from the remote_radio Quisk
 # -- Send mic sound to the remote_radio Quisk
 # These use sequential port numbers based on the TCP port number self.remote_ctl_base_port.
+# If you need to change the default base port number, you can edit the line in this file
+# that looks like (without the #):
 #
-# The TCP control interface is based on hamlib/rigctld protocol.  However, this is
-# not a generic interface to rigctld; it is separate from all other Quisk support for
-# hamlib/rigctl, and specific to communicating between quisk_hardware_control_head.py
-# and quisk_hardware_remote_radio.py.  It uses generic rigctl protocol when it can;
-# however, for best operation, Quisk requires certain information to be passed
-# that is awkward for generic rigctl protocol.  For example, frequency control uses a
-# "Raw" send_cmd (rigctl 'w'), to enable efficient conveyance of not just the tuning
-# frequency, but also VFO frequency, reason for change ("source"), and sometimes band.
+#   self.remote_ctl_base_port = 4585		# Remote Control base port
+#
+# Make sure to edit the corresponding line in remote_common.py to match ports!!
+#
+# You should be able to use the Quisk control_head along with any/all means of control that
+# you normally use to control Quisk, including serial ports, MIDI, and hamlib/rigctl interfaces.
 #
 # The remote_radio Quisk/computer is assumed to track the local control_head Quisk/computer;
-# no attempt is made to track the remote_radio Quisk's tuning frequency, mode, etc.
+# no attempt is made to verify the remote_radio Quisk's tuning frequency, mode, etc.
 # Snap-to Rx tuning for CW works on the control_head Quisk by virtue of graph/waterfall data
 # received from the remote_radio Quisk.
 #
@@ -63,7 +48,6 @@
 # configurable in terms of dit length, dit space, number of dits per burst, and pause between
 # bursts (phrases).  Search in this file for "DEBUG_CW_SEND_DITS" to find configurable variables.
 # To get log output from remote_radio end, in quisk_hardware_remote_radio.py, set DEBUG_CW_JITTER = 1.
-#
 
 from __future__ import print_function
 from __future__ import absolute_import
@@ -74,6 +58,7 @@ DEBUG = 0
 
 import socket, time, traceback, hmac, threading, select
 import _quisk as QS	# Access Quisk C functions via PyMethodDef QuiskMethods[] in quisk.c
+import wx
 
 from quisk_hardware_model import Hardware as BaseHardware
 
@@ -87,7 +72,6 @@ class ControlCommon(BaseHardware):	# This is the Hardware class for the control
     self.remote_ctl_connected = False
     self.remote_ctl_timestamp = None
     self.graph_data_port = self.remote_ctl_base_port + 1
-    self.graph_data_socket = None
     self.remote_radio_sound_port = self.remote_ctl_base_port + 2
     self.remote_mic_sound_port = self.remote_ctl_base_port + 3
     self.thread_lock = threading.Lock()
@@ -142,11 +126,6 @@ class ControlCommon(BaseHardware):	# This is the Hardware class for the control
       print('  Remote Control TCP socket already closed')
     self.remote_ctl_socket = None
     self.remote_ctl_connected = False
-    if self.graph_data_socket:
-      self.graph_data_socket.close()
-    else:
-      print('  Graph Data UDP socket already closed')
-    self.graph_data_socket = None
     QS.stop_control_head_remote_sound()
     self.app.main_frame.SetConfigText("Disconnected from remote radio " + self.conf.remote_radio_ip)
 
@@ -234,9 +213,10 @@ class ControlCommon(BaseHardware):	# This is the Hardware class for the control
         self.cw_poll_started = True
       elif key_down == 1:
         # Quisk startup with key down
-        t = f'Quisk is starting with CW key down!  Tx is on, and Rx is blocked until you release CW key.\n'
-        dlg = wx.MessageDialog(self.app.main_frame, t, "Quisk start, CW key down", style=wx.OK)
-        wx.CallAfter(dlg.ShowModal)
+        t = f'Quisk is starting with CW key down!  Tx is on, and Rx is blocked until you release CW key.'
+        print(t)
+        #dlg = wx.MessageDialog(self.app.main_frame, t, "Quisk start, CW key down", style=wx.OK)
+        #wx.CallAfter(dlg.ShowModal)
         self.cw_phrase_begin_ts = ts
         self.cw_poll_started = True
     if key_down != self.cw_keydown:
@@ -250,25 +230,6 @@ class ControlCommon(BaseHardware):	# This is the Hardware class for the control
       self.cw_phrase_end_ts = ts # End-of-cw-phrase-detection
       if DEBUG_CW_JITTER: print(f'{ts:10.4f} {key_down}, {cw_event_ts_msecs}')
 
-  def GetGraph(self, k, zoom, deltaf):
-    data = None
-    data1 = None
-    if self.graph_data_socket:
-      try:	# Read any data from the socket
-        data1, addr = self.graph_data_socket.recvfrom(4096*5)
-      except socket.timeout:	# This does not work
-        pass
-      except socket.error:	# Nothing to read
-        pass
-    if data1:
-      #print ("Receive graph", len(data1))
-      # Convert from packed block of int8 (bytes) to tuple/list of floats; reverse sign.
-      data = []
-      for d in data1:
-        data.append( - float(d))
-      #print ("Control GetGraph()", len(data))
-    return data
-
   def HeartBeat(self):	# Called at about 10 Hz by the main
     if self.closing:	# Don't try to connect if we are closing
       return
@@ -290,8 +251,8 @@ class ControlCommon(BaseHardware):	# This is the Hardware class for the control
       if not self.remote_ctl_connected:
         if DEBUG: print('Cannot send if not TCP connected:', text)
         return
-      if text[0:4] not in ("FREQ", "HEAR"):
-        print ("Control head RemoteCtlSend", text)
+      #if text[0:4] not in ("FREQ", "HEAR"):
+      #  print ("Control head RemoteCtlSend", text)
       #BMC if DEBUG: print('Send', text, end=' ')
       try:	# Patch thanks to Rolandas, LY0NAS
         self.remote_ctl_socket.sendall(text.encode('utf-8', errors='ignore'))
@@ -324,17 +285,6 @@ class ControlCommon(BaseHardware):	# This is the Hardware class for the control
         # S-meter text from remote_radio
         self.smeter_text = reply[2:]
         #print ("Receive smeter", reply[2:])
-      elif reply[0] in 'Xx':
-        print ("Control head received", reply)
-        # Window size info from remote radio Quisk; data_width must be == for good graph/waterfall display
-        args = reply[2:].split(';')
-        data_width = int(args[3])
-        if data_width != self.app.data_width:
-          t = f'Control head data_width {self.app.data_width} is different from Remote Radio data_width {data_width}.  \
-This will cause graph/waterfall data to be mis-aligned in Control Head display.  \
-Please configure main window widths to be identical, using YourRadio->Windows->Window width pixels\n'
-          dlg = wx.MessageDialog(self.app.main_frame, t, "data_width mismatch", style=wx.OK)
-          dlg.ShowModal()
       elif reply[0:6] == "TOKEN;":
         passw = self.app.local_conf.globals.get("remote_radio_password", "")
         passw = passw.strip()
@@ -342,20 +292,13 @@ Please configure main window widths to be identical, using YourRadio->Windows->W
           passw = passw.encode('utf-8')
           H = hmac.new(passw, reply[6:].encode('utf-8'), 'sha3_256')
           del passw
-          self.RemoteCtlSend("TOKEN;" + H.hexdigest() + "\n")
+          self.RemoteCtlSend("TOKEN;%s;%d\n" % (H.hexdigest(), self.app.data_width))
         else:
           print ("Error: Missing password on control head")
       elif reply[0:8] == "TOKEN_OK":
         self.app.main_frame.SetConfigText("Connected to remote radio " + self.conf.remote_radio_ip)
-        self.graph_data_socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
-        self.graph_data_socket.bind(('', self.graph_data_port))
-        self.graph_data_socket.settimeout(0.0)
-        QS.start_control_head_remote_sound(self.conf.remote_radio_ip, self.remote_radio_sound_port, self.remote_mic_sound_port)
-        # Send dimensions of control_head main window and graph/waterfall display and data widths
-        # to remote_radio; data_width must be identical in control_head and remote_radio in order
-        # for graph/waterfall to align properly.  Remote radio will respond with its dimensions.
-        t = f'X;{self.app.width};{self.app.height};{self.app.graph_width};{self.app.data_width}\n'
-        self.RemoteCtlSend(t)
+        QS.start_control_head_remote_sound(self.conf.remote_radio_ip, self.remote_radio_sound_port,
+                 self.remote_mic_sound_port, self.graph_data_port)
         self.CommonInit()	# Send initial parameters common to all radios
         self.RadioInit()	# Send initial parameters peculiar to a given radio
       elif reply[0:9] == "TOKEN_BAD":
@@ -395,3 +338,15 @@ Please configure main window widths to be identical, using YourRadio->Windows->W
     # Various buttons
     for idName in ("Mute", "NR2", "AGC", "Sqlch", "NB 1", "Notch", "Test 1", "Spot", "FDX", "PTT", "VOX"):
       self.RemoteCtlSend("%s;%d\n" % (idName, app.idName2Button[idName].GetIndex()))
+    # Menus
+    for menu in (app.NB_menu, app.split_menu, app.freedv_menu, app.smeter_menu):
+      if menu:
+        for nid in menu.id2data:
+          menu_item = menu.FindItemById(nid)
+          kind = menu_item.GetKind()
+          if kind == wx.ITEM_RADIO:
+            if menu_item.IsChecked():
+              self.RemoteCtlSend('MENU;%s;%s;1\n' % (menu.menu_name, menu_item.GetItemLabelText()))
+          elif kind == wx.ITEM_CHECK:
+            checked = menu_item.IsChecked()
+            self.RemoteCtlSend('MENU;%s;%s;%d\n' % (menu.menu_name, menu_item.GetItemLabelText(), int(checked)))
