@@ -20,10 +20,14 @@
 #include <sys/socket.h>
 #include <arpa/inet.h>
 #include <netinet/ip.h>
+#include <fcntl.h>
 #endif
 
 #include "../quisk.h"
 
+#define GRAPH_DATA_SCALE	163
+#define GRAPH_DATA_BLOCK	600
+
 #define REMOTE_DEBUG 0  //BMC TODO:  Make this a configuration option
 
 // For remote control sound; 2 sockets used at each end, but 4 needed to guide program flow
@@ -31,13 +35,36 @@ static SOCKET radio_sound_to_control_head_socket = INVALID_SOCKET;	// send radio
 static SOCKET radio_sound_from_remote_radio_socket = INVALID_SOCKET;	// receive radio sound from remote_radio
 static SOCKET mic_sound_from_control_head_socket = INVALID_SOCKET;	// receive mic sound from control_head
 static SOCKET mic_sound_to_remote_radio_socket = INVALID_SOCKET;	// send mic sound to remote_radio
+static SOCKET graph_data_to_control_head_socket = INVALID_SOCKET;	// send graph data to control_head
+static SOCKET graph_data_from_remote_radio_socket = INVALID_SOCKET;	// receive graph data from remote_radio
 static int sound_from_remote_socket_started = 0;			// sound (radio or mic) stream started from far end
 static int sound_to_remote_socket_started = 0;				// sound (radio or mic) stream started from near end
+static int control_head_data_width;					// app.data_width of control head for graph data
 static int packets_sent;
 static int packets_recd;
+static struct sockaddr_in AddrControlHeadGraph;				// socket address of control head graph data port
 
 static void send_remote_sound_socket(SOCKET * sock, complex double * cSamples, int nSamples);
 
+static void make_udp_socket(SOCKET * sock, int bufsize)
+{
+	const char enable = 1;
+
+	*sock = socket(PF_INET, SOCK_DGRAM, 0);
+	if (*sock != INVALID_SOCKET) {
+		setsockopt(*sock, SOL_SOCKET, SO_RCVBUF, (char *)&bufsize, sizeof(bufsize));
+		setsockopt(*sock, SOL_SOCKET, SO_REUSEADDR, &enable, sizeof(enable));
+#ifdef MS_WINDOWS
+        	unsigned long one = 1;
+		ioctlsocket(*sock, FIONBIO, &one);	// set non-blocking
+#else
+		int flags;
+		flags = fcntl(*sock, F_GETFL, 0);	// set non-blocking
+		fcntl(*sock, F_SETFL, flags | O_NONBLOCK);
+#endif
+	}
+}
+
 // Receive stereo 16-bit pcm sound via UDP
 // This code acts as UDP client for radio sound (on control head)
 int read_remote_radio_sound_socket(struct sound_dev* dev, complex double * cSamples, int nSamples)
@@ -85,10 +112,10 @@ int read_remote_radio_sound_socket(struct sound_dev* dev, complex double * cSamp
 #endif
 	// Signal far end (server) that we're ready (this sends our address/port to far end)
 	if (!sound_from_remote_socket_started) {
-		printf("read_remote_radio_sound_socket() sending 'rr', *sock = %u\n", (u_int)*sock);
+		QuiskPrintf("read_remote_radio_sound_socket() sending 'rr', *sock = %u\n", (u_int)*sock);
 		bytes = send(*sock, "rr\n", 3, 0);
 		if (bytes != 3)
-			printf("read_remote_radio_sound_socket(), sendto(): %s\n", strerror(errno));
+			QuiskPrintf("read_remote_radio_sound_socket(), sendto(): %s\n", strerror(errno));
 	}
 
 	// read all available packets, one per loop, place into jitter-absorbing circular buffer
@@ -115,13 +142,13 @@ int read_remote_radio_sound_socket(struct sound_dev* dev, complex double * cSamp
 				else
 					wr_len = rd_idx;
 #if REMOTE_DEBUG > 2 //BMC debug
-				printf("wrapping write at wrap_idx = %i, rd_idx = %i, wr_len = %li\n", wrap_idx, rd_idx, wr_len);
+				QuiskPrintf("wrapping write at wrap_idx = %i, rd_idx = %i, wr_len = %li\n", wrap_idx, rd_idx, wr_len);
 #endif
 			}
 #if REMOTE_DEBUG > 0
 			else {
 				double now = QuiskTimeSec();
-				printf("%f, jitbuf overflow, reduce recv fm %u to %u, wr_idx %u, rd_idx %u, samps_in_buf %u, remainder 0s\n",
+				QuiskPrintf("%f, jitbuf overflow, reduce recv fm %u to %u, wr_idx %u, rd_idx %u, samps_in_buf %u, remainder 0s\n",
 					now, (u_int)prior_pkt_size_max, (u_int)wr_len, wr_idx, rd_idx, samps_in_buf);
 			}
 #endif
@@ -132,7 +159,7 @@ int read_remote_radio_sound_socket(struct sound_dev* dev, complex double * cSamp
 		FD_ZERO (&fds);
 		FD_SET (*sock, &fds);
 		if (select(*sock + 1, &fds, NULL, NULL, &tm_wait) != 1) {
-			//BMC printf("read_remote_radio_sound_socket(): select returned %i\n", retval);
+			//BMC QuiskPrintf("read_remote_radio_sound_socket(): select returned %i\n", retval);
 			bytes = 0;
 			break;
 		}
@@ -140,7 +167,7 @@ int read_remote_radio_sound_socket(struct sound_dev* dev, complex double * cSamp
 		if (bytes < 0) {
 			int err = errno;
 			if (err != EAGAIN && err != EWOULDBLOCK)
-				printf("read_remote_radio_sound_socket(), recv(): %s\n", strerror(errno));
+				QuiskPrintf("read_remote_radio_sound_socket(), recv(): %s\n", strerror(errno));
 			break;
 		}
 		if (bytes > 0) {
@@ -156,13 +183,13 @@ int read_remote_radio_sound_socket(struct sound_dev* dev, complex double * cSamp
 		}
 	} // while(1)
 	if (!sound_from_remote_socket_started  && bytes > 0) {
-		printf("read_remote_radio_sound_socket(): Started! samps_in_buf = %i\n", bytes / 4);
+		QuiskPrintf("read_remote_radio_sound_socket(): Started! samps_in_buf = %i\n", bytes / 4);
 		sound_from_remote_socket_started = 1;
 	}
 	if (!prebuf_done) {
 		if (samps_in_buf >= prebuf_samples) {
 			prebuf_done = 1;
-			printf("read_remote_radio_sound_socket: Prebuf done, samps_in_buf = %i\n", samps_in_buf);
+			QuiskPrintf("read_remote_radio_sound_socket: Prebuf done, samps_in_buf = %i\n", samps_in_buf);
 		}
 	}
 
@@ -176,7 +203,7 @@ int read_remote_radio_sound_socket(struct sound_dev* dev, complex double * cSamp
 			//     set rd_idx so there is prebuf_samples number of samples in buffer.
 #if REMOTE_DEBUG > 0
 			double now = QuiskTimeSec();
-			printf("%f, Removing excess jitbuf samples: underrun_count %i, samps_in_buf %i, rd_idx %i, wrap_idx %i, wr_idx %i\n",
+			QuiskPrintf("%f, Removing excess jitbuf samples: underrun_count %i, samps_in_buf %i, rd_idx %i, wrap_idx %i, wr_idx %i\n",
 					now, underrun_count, samps_in_buf, rd_idx, wrap_idx, wr_idx);
 #endif
 			rd_idx = wr_idx - (prebuf_samples * 4);	// rd_idx = bytes; prebuf_samples = samples; 4 bytes per sample
@@ -185,7 +212,7 @@ int read_remote_radio_sound_socket(struct sound_dev* dev, complex double * cSamp
 			samps_in_buf = prebuf_samples;
 			underrun_count = 0;
 #if REMOTE_DEBUG > 0
-			printf("%f, Removed excess jitbuf samples: samps_in_buf %i, rd_idx %i, wrap_idx %i,\n",
+			QuiskPrintf("%f, Removed excess jitbuf samples: samps_in_buf %i, rd_idx %i, wrap_idx %i,\n",
 					now, samps_in_buf, rd_idx, wrap_idx);
 #endif
 		}
@@ -202,7 +229,7 @@ int read_remote_radio_sound_socket(struct sound_dev* dev, complex double * cSamp
 					tmp_idx = 0;
 					ptInt16 = (int16_t *)&buf[tmp_idx];
 #if 0 //BMC debug
-					printf("wrapping sidetone at wrap_idx = %i\n", wrap_idx);
+					QuiskPrintf("wrapping sidetone at wrap_idx = %i\n", wrap_idx);
 #endif
 				}
 			}
@@ -217,7 +244,7 @@ int read_remote_radio_sound_socket(struct sound_dev* dev, complex double * cSamp
 			double now = QuiskTimeSec();
 			numout = samps_in_buf;
 			underrun_count += (nSamples - samps_in_buf);
-			printf("%f, jitbuf low, reduce outsamps fm %i to %i, wr_idx %i, rd_idx %i, samps_in_buf %i, remainder 0s\n",
+			QuiskPrintf("%f, jitbuf low, reduce outsamps fm %i to %i, wr_idx %i, rd_idx %i, samps_in_buf %i, remainder 0s\n",
 				now, nSamples, numout, wr_idx, rd_idx, samps_in_buf);
 		}
 		ptInt16 = (int16_t *)&buf[rd_idx];
@@ -234,7 +261,7 @@ int read_remote_radio_sound_socket(struct sound_dev* dev, complex double * cSamp
 				rd_idx = 0;
 				ptInt16 = (int16_t *)&buf[rd_idx];
 #if REMOTE_DEBUG > 2 //BMC debug
-				printf("wrapping read at wrap_idx = %i\n", wrap_idx);
+				QuiskPrintf("wrapping read at wrap_idx = %i\n", wrap_idx);
 #endif
 			}
 		}
@@ -243,7 +270,7 @@ int read_remote_radio_sound_socket(struct sound_dev* dev, complex double * cSamp
 #if REMOTE_DEBUG > 1 //BMC debug -- per-call (perhaps multi-packet) statistics
 	if (lapcount != 1) {	// 1: 0 packets, 2: 1 packet, 3: 2 packets, etc.
 		double ts = QuiskTimeSec();
-		printf("%f: callcount = %lu, lapcount = %i, bytes = %i, nSamples = %i\n", ts, callcount, lapcount, bytes, nSamples);
+		QuiskPrintf("%f: callcount = %lu, lapcount = %i, bytes = %i, nSamples = %i\n", ts, callcount, lapcount, bytes, nSamples);
 	}
 #endif
 
@@ -253,11 +280,11 @@ int read_remote_radio_sound_socket(struct sound_dev* dev, complex double * cSamp
 		double delta = new_ts - prior_ts;
 		prior_ts = new_ts;
 #if REMOTE_DEBUG > 2 //BMC raw numbers
-		printf("%f: read_remote_radio_sound_socket CURRENT calls: %lu, samps_in %lu, samps_out %lu, nSamps %lu, samps_in_buf %i, deltasec %f\n",
+		QuiskPrintf("%f: read_remote_radio_sound_socket CURRENT calls: %lu, samps_in %lu, samps_out %lu, nSamps %lu, samps_in_buf %i, deltasec %f\n",
 			new_ts, callcount, sampcount_in, sampcount_out, nSamps, samps_in_buf, delta);
 #endif
 #if REMOTE_DEBUG > 1 //BMC raw numbers
-		printf("%f: read_remote_radio_sound_socket CURRENT RATES (HZ): calls %f, samps_in %f, samps_out %f, nSamps %f, samps_in_buf %i\n",
+		QuiskPrintf("%f: read_remote_radio_sound_socket CURRENT RATES (HZ): calls %f, samps_in %f, samps_out %f, nSamps %f, samps_in_buf %i\n",
 			new_ts, callcount / delta, sampcount_in / delta, sampcount_out / delta, nSamps / delta, samps_in_buf);
 #endif
 		if (bunchcount > 0) {	// skip the initial bunch; prebuf may distort some numbers(?)
@@ -269,14 +296,14 @@ int read_remote_radio_sound_socket(struct sound_dev* dev, complex double * cSamp
 		}
 		if (bunchcount % 10 == 0 && bunchcount > 0) {
 #if REMOTE_DEBUG > 1 //BMC
-			printf("%f: read_remote_radio_sound_socket SUMMARY calls: %f, samps_in %f, samps_out %f, nSamps %f, samps_in_buf %i, deltasec %f\n",
+			QuiskPrintf("%f: read_remote_radio_sound_socket SUMMARY calls: %f, samps_in %f, samps_out %f, nSamps %f, samps_in_buf %i, deltasec %f\n",
 				new_ts, callcount_total, sampcount_in_total, sampcount_out_total, nSamps_total, samps_in_buf, delta_total);
-			printf("%f: read_remote_radio_sound_socket SUMMARY RATES (HZ): calls %f, samps_in %f, samps_out %f, nSamps %f\n",
+			QuiskPrintf("%f: read_remote_radio_sound_socket SUMMARY RATES (HZ): calls %f, samps_in %f, samps_out %f, nSamps %f\n",
 				new_ts, callcount_total / delta_total, sampcount_in_total / delta_total,
 				sampcount_out_total / delta_total, nSamps_total / delta_total);
 #else
 			float latency = (float)samps_in_buf / 48000.0;
-			printf("%f: read_remote_radio_sound_socket samps_in_buf %i, latency %f sec\n", new_ts, samps_in_buf, latency);
+			QuiskPrintf("%f: read_remote_radio_sound_socket samps_in_buf %i, latency %f sec\n", new_ts, samps_in_buf, latency);
 #endif
 		}
 		bunchcount++;
@@ -315,10 +342,10 @@ int read_remote_mic_sound_socket(struct sound_dev* dev, complex double * cSample
 
 	// Signal far end (server) that we're ready (this sends our address/port to far end)
 	if (!sound_from_remote_socket_started) {
-		printf("read_remote_mic_sound_socket() sending 'rr', *sock = %u\n", (u_int)*sock);
+		QuiskPrintf("read_remote_mic_sound_socket() sending 'rr', *sock = %u\n", (u_int)*sock);
 		bytes = send(*sock, "rr\n", 3, 0);
 		if (bytes != 3)
-			printf("read_remote_mic_sound_socket(), sendto(): %s\n", strerror(errno));
+			QuiskPrintf("read_remote_mic_sound_socket(), sendto(): %s\n", strerror(errno));
 	}
 
 	// read all available packets, one per loop, place into jitter-absorbing circular buffer
@@ -336,7 +363,7 @@ int read_remote_mic_sound_socket(struct sound_dev* dev, complex double * cSample
 		FD_ZERO (&fds);
 		FD_SET (*sock, &fds);
 		if (select(*sock + 1, &fds, NULL, NULL, &tm_wait) != 1) {
-			//BMC printf("read_remote_mic_sound_socket(): select returned %i\n", retval);
+			//BMC QuiskPrintf("read_remote_mic_sound_socket(): select returned %i\n", retval);
 			bytes = 0;
 			break;
 		}
@@ -344,7 +371,7 @@ int read_remote_mic_sound_socket(struct sound_dev* dev, complex double * cSample
 		if (bytes < 0) {
 			int err = errno;
 			if (err != EAGAIN && err != EWOULDBLOCK)
-				printf("read_remote_mic_sound_socket(), recv(): %s\n", strerror(errno));
+				QuiskPrintf("read_remote_mic_sound_socket(), recv(): %s\n", strerror(errno));
 			break;
 		}
 		if (bytes > 0) {
@@ -356,7 +383,7 @@ int read_remote_mic_sound_socket(struct sound_dev* dev, complex double * cSample
 	} // while(1)
 
 	if (!sound_from_remote_socket_started  && bytes > 0) {
-		printf("read_remote_mic_sound_socket(): Started! samps_in_buf = %i\n", bytes / 4);
+		QuiskPrintf("read_remote_mic_sound_socket(): Started! samps_in_buf = %i\n", bytes / 4);
 		sound_from_remote_socket_started = 1;
 	}
 
@@ -430,12 +457,12 @@ static void send_remote_sound_socket(SOCKET * sock, complex double * cSamples, i
 		FD_ZERO (&fds);
 		FD_SET (*sock, &fds);
 		if ((retval = select(*sock + 1, &fds, NULL, NULL, &tm_wait)) != 1) {
-			//BMC printf("send_remote_sound_socket(): select returned %i\n", retval);
+			//BMC QuiskPrintf("send_remote_sound_socket(): select returned %i\n", retval);
 			return;
 		}
 		// Receive short msg, grab far end address
 		if ((recv_len = recvfrom(*sock, buf, RX_BUFFER_SIZE, 0, (struct sockaddr *) &far_addr, &addr_len)) == -1) {
-			printf("send_remote_sound_socket(), recvfrom(): %s\n", strerror(errno));
+			QuiskPrintf("send_remote_sound_socket(), recvfrom(): %s\n", strerror(errno));
 			return;
 		}
 		else if(recv_len > 0) {
@@ -443,9 +470,9 @@ static void send_remote_sound_socket(SOCKET * sock, complex double * cSamples, i
 				buf[RX_BUFFER_SIZE - 1] = '\n';
 			else
 				buf[recv_len] = '\n';
-			printf("send_remote_sound_socket(): recv_len = %i, %s", recv_len, buf);
+			QuiskPrintf("send_remote_sound_socket(): recv_len = %i, %s", recv_len, buf);
 			if (connect(*sock, (const struct sockaddr *)&far_addr, sizeof(far_addr)) != 0) {
-				printf("send_remote_sound_socket), connect(): %s\n", strerror(errno));
+				QuiskPrintf("send_remote_sound_socket), connect(): %s\n", strerror(errno));
 				close(*sock);
 				*sock = INVALID_SOCKET;
 			}
@@ -455,7 +482,7 @@ static void send_remote_sound_socket(SOCKET * sock, complex double * cSamples, i
 	}
 
 	if (nSamples > MAX_SAMPLES_FOR_REMOTE_SOUND) {
-		printf("send_remote_sound_socket():  nSamples %i > MAX_SAMPLES_FOR_REMOTE_SOUND 15,000, trimming to MAX\n", nSamples);
+		QuiskPrintf("send_remote_sound_socket():  nSamples %i > MAX_SAMPLES_FOR_REMOTE_SOUND 15,000, trimming to MAX\n", nSamples);
 		nSamples = MAX_SAMPLES_FOR_REMOTE_SOUND;
 	}
 
@@ -467,14 +494,14 @@ static void send_remote_sound_socket(SOCKET * sock, complex double * cSamples, i
 	if (udp_size > 0) {
 		sent = send(*sock, (char *)sound_lr, udp_size * 2, 0);
 		if (sent != udp_size * 2)
-			printf("send_remote_sound_socket(), send(): %s\n", strerror(errno));
+			QuiskPrintf("send_remote_sound_socket(), send(): %s\n", strerror(errno));
 #if REMOTE_DEBUG > 0 //BMC debug
 		else {
 			sampcount += sent/4;
 			packets_sent++;
 #if REMOTE_DEBUG > 1 //BMC debug
 			now = QuiskTimeSec();
-			printf("%f, send_remote_sound_socket(): now - prior = %f, samples = %u, sampcount = %lu\n",
+			QuiskPrintf("%f, send_remote_sound_socket(): now - prior = %f, samples = %u, sampcount = %lu\n",
 				now, now - prior_packet_ts, sent / 4, sampcount);
 			prior_packet_ts = now;
 #endif
@@ -484,7 +511,7 @@ static void send_remote_sound_socket(SOCKET * sock, complex double * cSamples, i
 #if REMOTE_DEBUG > 1 //BMC debug
 	else {
 		now = QuiskTimeSec();
-		printf("%f, send_remote_sound_socket(): nSamples = %i\n", now, nSamples);
+		QuiskPrintf("%f, send_remote_sound_socket(): nSamples = %i\n", now, nSamples);
 	}
 #endif
 #if REMOTE_DEBUG > 0 //BMC debug
@@ -493,8 +520,8 @@ static void send_remote_sound_socket(SOCKET * sock, complex double * cSamples, i
 		double delta = new_ts - prior_ts;
 		prior_ts = new_ts;
 #if REMOTE_DEBUG > 1 //BMC every 200
-		printf("send_remote_sound_socket CURRENT calls: %lu, samples %lu, deltasec %f\n", callcount, sampcount, delta);
-		printf("%f: send_remote_sound_socket CURRENT RATES (HZ): calls %f, samples %f\n", new_ts, callcount / delta, sampcount / delta);
+		QuiskPrintf("send_remote_sound_socket CURRENT calls: %lu, samples %lu, deltasec %f\n", callcount, sampcount, delta);
+		QuiskPrintf("%f: send_remote_sound_socket CURRENT RATES (HZ): calls %f, samples %f\n", new_ts, callcount / delta, sampcount / delta);
 #endif
 		if (bunchcount > 0) {	// skip the initial bunch; prebuf may distort some numbers(?)
 			callcount_total += callcount;
@@ -502,9 +529,9 @@ static void send_remote_sound_socket(SOCKET * sock, complex double * cSamples, i
 			delta_total += delta;
 		}
 		if (bunchcount % 10 == 0 && bunchcount > 0) {
-			printf("%f: send_remote_sound_socket SUMMARY calls: %f, samples %f, deltasec %f\n",
+			QuiskPrintf("%f: send_remote_sound_socket SUMMARY calls: %f, samples %f, deltasec %f\n",
 				new_ts, callcount_total, sampcount_total, delta_total);
-			printf("%f: send_remote_sound_socket SUMMARY RATES (HZ): calls %f, samples %f\n",
+			QuiskPrintf("%f: send_remote_sound_socket SUMMARY RATES (HZ): calls %f, samples %f\n",
 				new_ts, callcount_total / delta_total, sampcount_total / delta_total);
 		}
 		bunchcount++;
@@ -514,6 +541,93 @@ static void send_remote_sound_socket(SOCKET * sock, complex double * cSamples, i
 #endif
 }
 
+void send_graph_data(double * fft_avg, int fft_size, double zoom, double deltaf, int fft_sample_rate, double scale)
+{  // called by the sound thread
+	static double * pixels = NULL;
+	static int n_pixels = 0;
+	static int16_t sequence = 0;
+	int16_t buffer[GRAPH_DATA_BLOCK];
+	int16_t block;
+	int pixel_index, buffer_index;
+	double d1, d2;
+	ssize_t sent;
+
+	if (!sound_to_remote_socket_started || !control_head_data_width)
+		return;
+	if (control_head_data_width > n_pixels) {
+		n_pixels = control_head_data_width;
+		if (pixels)
+			free(pixels);
+		pixels = (double *)malloc(n_pixels * sizeof(double));
+	}
+	copy2pixels(pixels, control_head_data_width, fft_avg, fft_size, zoom, deltaf, fft_sample_rate);
+	// Send multiple 16-bit data blocks: sequence, block number, 16-bit graph data
+	block = 0;
+	pixel_index = 0;
+	while (pixel_index < control_head_data_width) {
+		buffer[0] = sequence;
+		buffer[1] = block;
+		buffer_index = 2;
+		while (buffer_index < GRAPH_DATA_BLOCK && pixel_index < control_head_data_width) {
+			d1 = pixels[pixel_index++];
+			if (fabs(d1) < 1e-40)	// avoid log10(0)
+				d1 = 1E-40;
+			d2 = 20.0 * log10(d1) - scale;
+			if (d2 < -200)
+				d2 = -200;
+			else if (d2 > 0)
+				d2 = 0;
+			buffer[buffer_index++] = (int16_t)lround(d2 * GRAPH_DATA_SCALE);
+		}
+		sent = sendto(graph_data_to_control_head_socket, (const char *)buffer, buffer_index * 2, 0,
+                     	  	(const struct sockaddr *)&AddrControlHeadGraph, sizeof(AddrControlHeadGraph));
+		if (sent != buffer_index * 2)
+			QuiskPrintf("send_graph_data(), send(): %s\n", strerror(errno));
+		block++;
+	}
+	sequence = (sequence + 1) & 0xFF;	// 8 bits
+}
+
+int receive_graph_data(double * fft_avg)
+{  // called by the sound thread
+	int i, i1, i2;
+	ssize_t count;
+	int16_t buffer[GRAPH_DATA_BLOCK];
+	int16_t block;
+	static int16_t * pixels = NULL;
+	static int n_pixels = 0;
+	static int total = 0;
+	static int16_t sequence = 0;
+
+	if (n_pixels < data_width) {
+		n_pixels = data_width;
+		if (pixels)
+			free(pixels);
+		pixels = (int16_t *)malloc(n_pixels * sizeof(int16_t));
+	}
+	count = recv(graph_data_from_remote_radio_socket, (char *)buffer, GRAPH_DATA_BLOCK * 2, 0);
+	count /= 2;	// convert to int16_t
+	if (count > 2) {
+		if (buffer[0] != sequence) {	// new graph data
+			sequence = buffer[0];
+			total = 0;
+		}
+		block = buffer[1];
+		count -= 2;	// number of 16-bit graph data items
+		i1 = block * (GRAPH_DATA_BLOCK - 2);
+		i2 = i1 + count;
+		if (i1 >= 0 && i2 <= data_width) {
+			memcpy(pixels + i1, buffer + 2, count * 2);
+			total += count;
+			if (total == data_width) {
+				for (i = 0; i < data_width; i++)
+					fft_avg[i] = (double)pixels[i] / GRAPH_DATA_SCALE;
+				return data_width;
+			}
+		}
+	}
+	return 0;
+}
 
 
 static int start_winsock()
@@ -523,7 +637,7 @@ static int start_winsock()
 	WSADATA wsaData;
 
 	if (WSAStartup(wVersionRequested, &wsaData) != 0) {
-		printf("start_winsock(): %s\n", strerror(errno));
+		QuiskPrintf("start_winsock(): %s\n", strerror(errno));
 		return 0;		// failure to start winsock
 	}
 #endif
@@ -543,7 +657,7 @@ static void open_remote_sound_server(SOCKET * sock, char * ip, int port, int snd
 #endif
 
 	if (!start_winsock()) {
-		printf("open_remote_sound_server for %s: Failure to start WinSock\n", name);
+		QuiskPrintf("open_remote_sound_server for %s: Failure to start WinSock\n", name);
 		return;
 	}
 
@@ -561,16 +675,16 @@ static void open_remote_sound_server(SOCKET * sock, char * ip, int port, int snd
 		bind_addr.sin_port = htons(port);
 		bind_addr.sin_addr.s_addr = htonl(INADDR_ANY);
 		if (bind(*sock, (const struct sockaddr *)&bind_addr, sizeof(bind_addr)) != 0) {
-			printf("open_remote_sound_server(), bind(): %s\n", strerror(errno));
+			QuiskPrintf("open_remote_sound_server(), bind(): %s\n", strerror(errno));
 			close(*sock);
 			*sock = INVALID_SOCKET;
 		}
 	}
 	if (*sock == INVALID_SOCKET) {
-		printf("open server %s: Failure to open socket\n", name);
+		QuiskPrintf("open server %s: Failure to open socket\n", name);
 	}
 	else {
-		printf("open server %s: opened socket %s port %i\n", name, ip, port);
+		QuiskPrintf("open server %s: opened socket %s port %i\n", name, ip, port);
 	}
 }
 
@@ -581,7 +695,7 @@ static void open_remote_sound_client(SOCKET * sock, char * ip, int port, int snd
 	const char enable = 1;	// for sockopt
 
 	if (!start_winsock()) {
-		printf("open_remote_sound_client for %s: Failure to start WinSock\n", name);
+		QuiskPrintf("open_remote_sound_client for %s: Failure to start WinSock\n", name);
 		return;
 	}
 
@@ -604,10 +718,10 @@ static void open_remote_sound_client(SOCKET * sock, char * ip, int port, int snd
 		}
 	}
 	if (*sock == INVALID_SOCKET) {
-		printf("open client %s: Failure to open socket\n", name);
+		QuiskPrintf("open client %s: Failure to open socket\n", name);
 	}
 	else {
-		printf("open client %s: opened socket %s port %i\n", name, ip, port);
+		QuiskPrintf("open client %s: opened socket %s port %i\n", name, ip, port);
 	}
 }
 
@@ -620,10 +734,10 @@ static void close_remote_sound_socket(SOCKET * sock, char * name)
 #ifdef MS_WINDOWS
 		WSACleanup();
 #endif
-		printf("%s: closed socket\n", name);
+		QuiskPrintf("%s: closed socket\n", name);
 	}
 	else {
-		printf("%s: socket already closed\n", name);
+		QuiskPrintf("%s: socket already closed\n", name);
 	}
 }
 
@@ -633,12 +747,14 @@ PyObject * quisk_start_control_head_remote_sound(PyObject * self, PyObject * arg
 {
 	int radio_sound_port;
 	int mic_sound_port;
+	int graph_data_port;
 	int sndsize = 48000;
 	char * remote_radio_ip;	// IP address of far end
 	char * name;
 	SOCKET * sock;
+	struct sockaddr_in bind_addr;
 
-	if (!PyArg_ParseTuple (args, "sii", &remote_radio_ip, &radio_sound_port, &mic_sound_port))
+	if (!PyArg_ParseTuple (args, "siii", &remote_radio_ip, &radio_sound_port, &mic_sound_port, &graph_data_port))
 		return NULL;
 
 	name = "radio sound from remote_radio";
@@ -649,6 +765,19 @@ PyObject * quisk_start_control_head_remote_sound(PyObject * self, PyObject * arg
 	sock = &mic_sound_to_remote_radio_socket;
 	open_remote_sound_server(sock, remote_radio_ip, mic_sound_port, sndsize, name);
 
+	make_udp_socket(&graph_data_from_remote_radio_socket, 1024 * 8);
+	if (graph_data_from_remote_radio_socket != INVALID_SOCKET) {
+		memset((char *) &bind_addr, 0, sizeof(bind_addr));
+		bind_addr.sin_family = AF_INET;
+		bind_addr.sin_port = htons(graph_data_port);
+		bind_addr.sin_addr.s_addr = htonl(INADDR_ANY);
+		if (bind(graph_data_from_remote_radio_socket, (const struct sockaddr *)&bind_addr, sizeof(bind_addr)) != 0) {
+			QuiskPrintf("graph data bind: %s\n", strerror(errno));
+			close(graph_data_from_remote_radio_socket);
+			graph_data_from_remote_radio_socket = INVALID_SOCKET;
+		}
+	}
+
 	packets_sent = 0;
 	packets_recd = 0;
 
@@ -672,10 +801,15 @@ PyObject * quisk_stop_control_head_remote_sound(PyObject * self, PyObject * args
 	sock = &mic_sound_to_remote_radio_socket;
 	close_remote_sound_socket(sock, name);
 
+	if (graph_data_to_control_head_socket != INVALID_SOCKET) {
+		close(graph_data_to_control_head_socket);
+		graph_data_to_control_head_socket = INVALID_SOCKET;
+	}
+
 	sound_from_remote_socket_started = 0;	// reset for next time
 	sound_to_remote_socket_started = 0;
 
-	printf("total packets sent = %i, recd = %i\n", packets_sent, packets_recd);
+	QuiskPrintf("total packets sent = %i, recd = %i\n", packets_sent, packets_recd);
 
 	return Py_None;
 }
@@ -686,14 +820,25 @@ PyObject * quisk_start_remote_radio_remote_sound(PyObject * self, PyObject * arg
 {
 	int radio_sound_port;
 	int mic_sound_port;
+	int graph_data_port;
 	int sndsize = 48000;
 	char * control_head_ip;	// IP address of far end
 	char * name;
 	SOCKET * sock;
 
-	if (!PyArg_ParseTuple (args, "sii", &control_head_ip, &radio_sound_port, &mic_sound_port))
+	if (!PyArg_ParseTuple (args, "siiii", &control_head_ip, &radio_sound_port, &mic_sound_port,
+                     &graph_data_port, &control_head_data_width))
 		return NULL;
 
+	memset((char *) &AddrControlHeadGraph, 0, sizeof(AddrControlHeadGraph));
+	AddrControlHeadGraph.sin_family = AF_INET;
+	AddrControlHeadGraph.sin_port = htons(graph_data_port);
+#ifdef MS_WINDOWS
+	AddrControlHeadGraph.sin_addr.S_un.S_addr = inet_addr(control_head_ip);
+#else
+	inet_aton(control_head_ip, &AddrControlHeadGraph.sin_addr);
+#endif
+
 	name = "radio sound to control_head";
 	sock = &radio_sound_to_control_head_socket;
 	open_remote_sound_server(sock, control_head_ip, radio_sound_port, sndsize, name);
@@ -702,6 +847,8 @@ PyObject * quisk_start_remote_radio_remote_sound(PyObject * self, PyObject * arg
 	sock = &mic_sound_from_control_head_socket;
 	open_remote_sound_client(sock, control_head_ip, mic_sound_port, sndsize, name);
 
+	make_udp_socket(&graph_data_to_control_head_socket, 1024 * 8);
+
 	packets_sent = 0;
 	packets_recd = 0;
 
@@ -725,10 +872,16 @@ PyObject * quisk_stop_remote_radio_remote_sound(PyObject * self, PyObject * args
 	sock = &mic_sound_from_control_head_socket;
 	close_remote_sound_socket(sock, name);
 
+	if (graph_data_from_remote_radio_socket != INVALID_SOCKET) {
+		close(graph_data_from_remote_radio_socket);
+		graph_data_from_remote_radio_socket = INVALID_SOCKET;
+	}
+
 	sound_from_remote_socket_started = 0;	// reset for next time
 	sound_to_remote_socket_started = 0;
+	control_head_data_width = 0;
 
-	printf("total packets sent = %i, recd = %i\n", packets_sent, packets_recd);
+	QuiskPrintf("total packets sent = %i, recd = %i\n", packets_sent, packets_recd);
 
 	return Py_None;
 }
