@@ -2,41 +2,19 @@
 #    RUNNING QUISK TO CONTROL ATTACHED ACTUAL/REAL RADIO HARDWARE.
 # IT CONNECTS BY NETWORK TO A SEPARATE "CONTROL_HEAD" COMPUTER ALSO RUNNING QUISK.
 #
-# This software is Copyright (C) 2021 by Ben Cahill and 2006-2021 by James C. Ahlstrom.,
+# This software is Copyright (C) 2021-2022 by Ben Cahill and 2006-2022 by James C. Ahlstrom.,
 # and is licensed for use under the GNU General Public License (GPL).
 # See http://www.opensource.org.
 # Note that there is NO WARRANTY AT ALL.  USE AT YOUR OWN RISK!!
 #
-# This file, quisk_hardware_remote_radio.py, allows a radio-less (control_head) Quisk,
+# This file, remote_common.py, along with a radio-specific file, e.g.
+# remote_softrock.py, remote_hermes.py, or similar, allows a radio-less (control_head) Quisk,
 # running on a separate computer, to connect to this (remote_radio) instance of Quisk,
 # which has an actual, real radio attached.
 #
-# The control_head Quisk must use quisk_hardware_control_head.py to communicate
-# with this remote_radio Quisk via network connection.
-#
-# Set up the remote_radio Quisk by creating a new Radio, then configuring that Radio's
-# Hardware (tab) --> "Hardware File Path" to point to this file (use the "Change"
-# button to help you find it:  quisk_hardware_remote_radio.py).  Edit this file to:
-#
-# 1).  Import your usual radio's quisk_hardware_*.py file as BaseHardware, so that
-#      quisk_hardware_remote_radio.py is "installed on top of" your usual radio,
-#      e.g. for SoftRock USB:
-#
-#      from softrock.hardware_usb import Hardware as BaseHardware
-#
-#      NOTE:  You may need to consider how the two radio "layers" interact.
-#             e.g. for Softrock, quisk_hardware_remote_radio.py replaces the
-#             HeartBeat() and PollCwKey() functions in Softrock's hardware_usb.py
-#             This is fine, since Softrock's HeartBeat() is empty, and we won't use the
-#             Softrock hardware's built-in paddle port for keying because we will get
-#             CW keying from the control_head Quisk computer.  However, other radios
-#             might possibly require a call from quisk_hardware_remote_radio.py
-#             to invoke the corresponding function in the BaseHardware, e.g.
-#             call BaseHardware.PollCwKey() from within this file's PollCwKey()
-#
-# 2).  Agree with the control_head computer regarding range of ports used, e.g.:
-#
-#      self.remote_ctl_base_port = 4585		# Remote Control base port
+# The control_head Quisk must use control_common.py, along with a radio-specific file,
+# e.g. control_softrock.py, control_hermes.py, or similar, to communicate with this
+# remote_radio Quisk via a network connection.
 #
 # This remote_radio computer should be set up with a static IP address, so that you know
 # where to point the control_head.  The control head computer may, however, use dynamic
@@ -44,25 +22,22 @@
 # remote control connection is made.
 #
 # The main control interface between control_head and remote_radio is via a TCP port;
-# this uses very low bandwidth.  All functional control, including CW keyihg, is done via this port.
+# this uses very low bandwidth.  All functional control, including CW keying, is done via this port.
 #
 # There are 3 additional ports, all UDP, using low to moderate bandwidth:
 # -- Send graph/waterfall data to the control_head Quisk
 # -- Send radio sound to the control_head Quisk
 # -- Receive mic sound from the control_head Quisk
 # These use sequential port numbers based on the TCP port number self.remote_ctl_base_port.
+# If you need to change the default base port number, you can edit the line in this file
+# that looks like (without the #):
 #
-# The TCP control interface is based on hamlib/rigctld protocol.  However, this is
-# not a generic interface to rigctld; it is separate from all other Quisk support for
-# hamlib/rigctl, and specific to communicating between quisk_hardware_control_head.py
-# and quisk_hardware_remote_radio.py.  It uses generic rigctl protocol when it can;
-# however, for best operation, Quisk requires certain information to be passed
-# that is awkward for generic rigctl protocol.  For example, frequency control uses a
-# "Raw" send_cmd (rigctl 'w'), to enable efficient conveyance of not just the tuning
-# frequency, but also VFO frequency, reason for change ("source"), and sometimes band.
+#   self.remote_ctl_base_port = 4585		# Remote Control base port
 #
-# The remote_radio Quisk/computer is assumed to track the local control_head Quisk/computer;
-# no attempt is made by the control_head to track the remote_radio Quisk's tuning frequency, mode, etc.
+# Make sure to edit the corresponding line in control_common.py to match ports!!
+#
+# This remote_radio Quisk/computer is assumed to track the connected control_head Quisk/computer;
+# no attempt is made by the control_head to verify the remote_radio Quisk's tuning frequency, mode, etc.
 # Snap-to Rx tuning for CW works on the control_head Quisk by virtue of graph/waterfall data
 # received from the remote_radio Quisk.
 #
@@ -71,7 +46,6 @@
 # (e.g. to check thread scheduling behavior), set DEBUG_CW_JITTER = 2.
 # To send "perfect" bursts of CW dits from the control_head, set DEBUG_CW_SEND_DITS = 1
 # in the control_head's quisk_hardware_control_head.py.
-#
 
 DEBUG_CW_JITTER = 0
 DEBUG = 0
@@ -96,7 +70,6 @@ class Remot:	# Remote comtrol base class
     self.remote_ctl_heartbeat_ts = None
     self.remote_ctl_heartbeat_timeout = 10.0	# Close our connection if we don't hear heartbeat from Control Head
     self.graph_data_port = self.remote_ctl_base_port + 1
-    self.graph_data_socket = None
     self.remote_radio_sound_port = self.remote_ctl_base_port + 2
     self.remote_mic_sound_port = self.remote_ctl_base_port + 3
 
@@ -165,9 +138,6 @@ class Remot:	# Remote comtrol base class
     if self.remote_ctl_connection:
       self.remote_ctl_connection.close()
     self.remote_ctl_connection = None
-    if self.graph_data_socket:
-      self.graph_data_socket.close()
-    self.graph_data_socket = None
     QS.stop_remote_radio_remote_sound()
 
   def RemoteCtlSend(self, text):
@@ -234,7 +204,7 @@ class Remot:	# Remote comtrol base class
       self.cmd_text = cmd_text
       args = cmd_text.split(';')	# Split at ';' because some control names have blanks
       command = args[0]
-      param = args[1:]
+      params = args[1:]
       # TOKEN
       if self.token:
         if command == "TOKEN":
@@ -248,12 +218,12 @@ class Remot:	# Remote comtrol base class
           del passw
           if hmac.compare_digest(H.hexdigest(), args[1]):
             self.token = None
-            print ("Security challenge passed")
+            print ("Security challenge passed", args[2])
+            self.control_head_data_width = int(args[2])
             self.RemoteCtlSend("TOKEN_OK\n")
             self.remote_ctl_heartbeat_ts = time.time()
-            self.graph_data_socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
-            self.graph_data_socket.settimeout(0.0)
-            QS.start_remote_radio_remote_sound(self.control_head_ip, self.remote_radio_sound_port, self.remote_mic_sound_port)
+            QS.start_remote_radio_remote_sound(self.control_head_ip, self.remote_radio_sound_port,
+                       self.remote_mic_sound_port, self.graph_data_port, self.control_head_data_width)
           else:
             time.sleep(1)
         elif time.time() - self.token_time > 5:
@@ -295,10 +265,10 @@ class Remot:	# Remote comtrol base class
         freq = int(freq)
         vfo = int(vfo)
         rxFreq = int(rxFreq)
-        new_rxfreq = (rxFreq != self.app.rxFreq)
-        self.app.rxFreq = rxFreq
+        if rxFreq == self.app.rxFreq:
+          rxFreq = None
         var_decim_index = int(var_decim_index)
-        self.app.ChangeDisplayFrequency(freq - vfo, vfo, new_rxfreq)
+        self.app.ChangeHwFrequency(freq - vfo, vfo, source, band, None, rxFreq)
         if source == "NewDecim":
           self.app.config_screen.config.btn_decimation.SetSelection(var_decim_index)
           sample_rate = self.VarDecimSet(var_decim_index)
@@ -333,7 +303,7 @@ class Remot:	# Remote comtrol base class
             self.cw_next_event_ts = None
             self.cw_next_keydown = None
             self.cw_key_down = 0
-            QS.set_cwkey(0)
+            QS.set_remote_cwkey(0)
           else:
             # Begin new cw phrase; any prior phrase should be done by now.
             # Set up first cw event to be ready to execute.
@@ -353,21 +323,10 @@ class Remot:	# Remote comtrol base class
           continue	# No need to call handler, as rxFreq and txFreq are already handled
         menu = getattr(self.app, menu_name)
         nid = menu.item_text2id[item_text]
-        menu.Handler(None, nid)
         menu_item = menu.FindItemById(nid)
         if menu_item.IsCheckable():
           menu_item.Check(int(checked))
-      # Main window and graph/waterfall dimensions; graph data_width of control_head and remote_radio must match.
-      elif command == 'X':
-        ctl_width = int(param[0])
-        ctl_height = int(param[1])
-        ctl_graph_width = int(param[2])
-        ctl_data_width = int(param[3])
-        if self.app.data_width != ctl_data_width:
-          print('WARNING:  Control head graph data width', ctl_data_width, 'different from remote radio graph data width', self.app.data_width)
-        # Return remote_radio window and graph dimensions to control_head, so it can verify graph data_width is same
-        t = f'X;{self.app.width};{self.app.height};{self.app.graph_width};{self.app.data_width}\n'
-        self.RemoteCtlSend(t)
+        menu.Handler(None, nid)
       # Help buttons ".."
       elif command == '..':
         pass
@@ -388,8 +347,8 @@ class Remot:	# Remote comtrol base class
         self.cw_next_event_ts, self.cw_next_keydown = self.cw_event_queue.popleft()
         if DEBUG_CW_JITTER: print(f'{ts:10.4f} queue len: {cw_queue_len}, popping: {self.cw_next_keydown} {self.cw_next_event_ts:10.4f}')
       if ts >= self.cw_next_event_ts:
-        if DEBUG_CW_JITTER: print(f'{ts:10.4f} set_cwkey: {self.cw_next_keydown}')
-        QS.set_cwkey(self.cw_next_keydown)
+        if DEBUG_CW_JITTER: print(f'{ts:10.4f} set_remote_cwkey: {self.cw_next_keydown}')
+        QS.set_remote_cwkey(self.cw_next_keydown)
         self.cw_key_down = self.cw_next_keydown
         cw_queue_len = len(self.cw_event_queue)
         if cw_queue_len > 0:
@@ -406,22 +365,4 @@ class Remot:	# Remote comtrol base class
     self.cw_next_event_ts = None
     self.cw_next_keydown = None
     self.cw_key_down = 0
-    QS.set_cwkey(0)
-
-  def GetGraph(self, k, zoom, deltaf):
-    data = QS.get_graph(k, zoom, deltaf)	# get FFT data
-    if data:
-      self.RemoteCtlSend("M;%s\n" % self.app.smeter.GetLabel())
-      if self.graph_data_socket:
-        # Convert from tuple of floats to packed array of int8 (bytes)
-        # Floats are -200.0 to 0.0; change sign to +, reverse on reception
-        barray = bytearray()
-        for d in data:
-          barray.append(int( - d + 0.5))
-        try:
-          self.graph_data_socket.sendto(barray, (self.control_head_ip, self.graph_data_port))
-        except socket.timeout:	# This does not work
-          pass
-        except socket.error:	# No write capacity(?) ... this avoids Python throwing error
-          pass
-    return data
+    QS.set_remote_cwkey(0)
