@@ -27,7 +27,7 @@
 // 3 == Copy transmit audio to the output.
 #define TEST_AUDIO	0
 
-#define QUISK_CWKEY_DOWN	(quisk_hardware_cwkey || quisk_serial_key_down || quisk_midi_cwkey)
+#define QUISK_CWKEY_DOWN	(quisk_hardware_cwkey || quisk_serial_key_down || quisk_midi_cwkey || quisk_remote_cwkey)
 
 #ifdef MS_WINDOWS
 #define QUISK_SHUT_RD	SD_RECEIVE
@@ -258,6 +258,8 @@ extern PyObject * quisk_start_control_head_remote_sound(PyObject * self, PyObjec
 extern PyObject * quisk_stop_control_head_remote_sound(PyObject * self, PyObject * args);
 extern PyObject * quisk_start_remote_radio_remote_sound(PyObject * self, PyObject * args);
 extern PyObject * quisk_stop_remote_radio_remote_sound(PyObject * self, PyObject * args);
+extern int receive_graph_data(double * fft_avg);
+extern void send_graph_data(double * fft_avg, int fft_size, double zoom, double deltaf, int fft_sample_rate, double scale);
 
 void QuiskWavClose(struct QuiskWav *);
 int QuiskWavWriteOpen(struct QuiskWav *, char *, short, short, short, int, double);
@@ -269,6 +271,7 @@ void QuiskWavReadC(struct QuiskWav *, complex double *, int);
 void QuiskWavReadD(struct QuiskWav *, double *, int);
 void QuiskMeasureRate(const char *, int, int, int);
 void quisk_record_audio(struct wav_file *, complex double *, int);
+void copy2pixels(double * pixels, int n_pixels, double * fft, int fft_size, double zoom, double deltaf, double rate);
 
 extern struct sound_conf quisk_sound_state, * pt_quisk_sound_state;
 extern int mic_max_display;		// display value of maximum microphone signal level
@@ -305,6 +308,7 @@ extern int quisk_serial_key_down;		// The state of the serial port CW key
 extern int quisk_serial_ptt;			// The state of the serial port PTT
 extern int quisk_hardware_cwkey;		// The state of the hardware CW key from UDP or USB
 extern int quisk_midi_cwkey;			// The state of the MIDI CW key
+extern int quisk_remote_cwkey;			// The state of the remote CW key
 extern int quisk_sidetoneFreq;			// Frequency in hertz for the sidetone
 extern int quisk_active_sidetone;		// Whether and how to generate a sidetone
 extern int quisk_isFDX;				// Are we in full duplex mode?
