@@ -862,6 +862,19 @@ Hermes_BandDictEnTx = False
 #default_rf_gain = 11
 
 
+################ Receivers Control Head, Use Quisk to control a remote radio. No real hardware is used here.
+## hardware_file_name		Hardware file path, rfile
+# Quisk can be used as a control head to control a real radio located remotely.
+# This file contains the control logic for each remote radio.
+hardware_file_name = 'ac2yd/control_softrock.py'
+#hardware_file_name = 'ac2yd/control_hermes.py'
+
+## widgets_file_name		Widget file path, rfile
+# This file adds additional controls for the radio. Choose the correct file (if any) for the remote radio.
+widgets_file_name = ''
+#widgets_file_name = 'hermes/quisk_widgets.py'
+
+
 ################ Sound
 # Playback devices:
 #    name_of_sound_play      Play radio sound on speakers or headphones
@@ -1306,7 +1319,7 @@ dxClPassword = ''
 dxClExpireTime = 20
 
 ## IQ_Server_IP         Pulse server IP address, text
-#IP Adddress for remote PulseAudio IQ server.
+#IP Address for remote PulseAudio IQ server.
 IQ_Server_IP = ""
 
 ## hamlib_ip            IP address for Hamlib Rig 2, text
@@ -1362,6 +1375,20 @@ win_hamlib_com2_name = ""
 #win_hamlib_com2_name = "COM15"
 #win_hamlib_com2_name = "COM16"
 
+## remote_radio_ip         IP address of remote radio, text
+# Quisk can be used as a control head to control a real radio located remotely.
+# This is the IP adddress of the remote radio.
+remote_radio_ip = ""
+#remote_radio_ip = "192.168.1.56"
+
+## remote_radio_password         Password for remote radio, text
+# Quisk can be used as a control head to control a real radio located remotely.
+# For security it requires the same password to be entered on both computers.
+# Enter a fairly long pass phrase of 20 or more characters.
+# It is only necessary to enter it once on each computer.
+remote_radio_password = ""
+
+
 
 hamlib_com1_name = ""
 hamlib_com2_name = ""
@@ -1641,7 +1668,7 @@ cwTone = 600
 
 ## use_sidetone				Use sidetone, integer choice
 # This controls whether Quisk will display a sidetone volume control
-# and gererate a CW sidetone.
+# and generate a CW sidetone.
 use_sidetone = 0
 #use_sidetone = 1
 
