@@ -261,9 +261,9 @@ class SliderBoxH:
     if event:
       event.Skip()
       if self.handler:
-        self.handler(event)
         if application.remote_control_head:
           application.Hardware.RemoteCtlSend(f'{self.idName};{self.GetValue()}\n')
+        self.handler(event)
     if self.display:
       t = self.text % (self.slider.GetValue() * self.scale)
     else:
@@ -342,9 +342,9 @@ class SliderBoxV(wx.BoxSizer):
     self.Add(self.text_ctrl, 0, 0)
     self.Add(self.slider, 1, 0)
   def OnScroll(self, event):
-    self.handler(event)
     if application.remote_control_head:
       application.Hardware.RemoteCtlSend(f'{self.idName};{self.GetValue()}\n')
+    self.handler(event)
   def Change(self, event):
     event.Skip()
     self.text_ctrl.SetLabel(str(self.slider.GetValue()))
@@ -583,9 +583,9 @@ class QuiskPushbutton(QuiskButtons, wx.lib.buttons.GenButton):
       self.Bind(wx.EVT_RIGHT_UP, self.OnRightUp)
   def OnButton(self, event):
     if self.command:
-      self.command(event)
       if application.remote_control_head:
         application.Hardware.RemoteCtlSend(self.idName + ';1\n')
+      self.command(event)
   def OnRightDown(self, event):
     self.direction = -1
     self.OnLeftDown(event) 
@@ -700,9 +700,9 @@ class QuiskCheckbutton(QuiskButtons, wx.lib.buttons.GenToggleButton):
       return 0
   def OnButton(self, event):
     if self.command:
-      self.command(event)
       if application.remote_control_head:
         application.Hardware.RemoteCtlSend(f'{self.idName};{self.GetIndex()}\n')
+      self.command(event)
   def OnRightDown(self, event):
     self.direction = -1
     self.OnLeftDown(event) 
@@ -964,11 +964,12 @@ class WrapMenu(WrapControl):
     self.Add(button, 1, flag=wx.ALIGN_CENTER_VERTICAL)
     b = QuiskBitmapButton(button.GetParent(), self.OnPopButton, _bitmap_menupop)
     self.Add(b, 0, flag=wx.ALIGN_CENTER_VERTICAL|wx.LEFT, border=2)
+    self.pop_button = b
   def OnPopButton(self, event):
     if self.on_open:
       self.on_open(self.menu)
     pos = (5, 5)
-    self.button.PopupMenu(self.menu, pos)
+    self.pop_button.PopupMenu(self.menu, pos)
 
 class WrapSlider(WrapControl):
   def __init__(self, button, command, slider_value=0, slider_min=0, slider_max=1000, display=False, wintype=''):
@@ -1030,11 +1031,11 @@ class WrapSlider(WrapControl):
       self.button.SetLabel(str(value))
       self.button.Refresh()
     if self.command:
+      if application.remote_control_head:
+        application.Hardware.RemoteCtlSend(f'{self.idName}Slider;{value}\n')
       event = wx.PyEvent()
       event.SetEventObject(self.button)
       self.command(event)
-      if application.remote_control_head:
-        application.Hardware.RemoteCtlSend(f'{self.idName}Slider;{value}\n')
   def SetSlider(self, value=None, value_off=None, value_on=None):
     if value is not None:
       self.button.slider_value = value
@@ -1128,9 +1129,9 @@ class QuiskCycleCheckbutton(QuiskCheckbutton):
     else:
       self.direction = 0
     if self.command:
-      self.command(event)
       if application.remote_control_head:
         application.Hardware.RemoteCtlSend(f'{self.idName};{self.GetIndex()}\n')
+      self.command(event)
   def OnRightDown(self, event):		# Move left in the list of labels
     if not self.is_radio or self.GetValue():
       self.index -= 1
@@ -1139,18 +1140,18 @@ class QuiskCycleCheckbutton(QuiskCheckbutton):
       self.SetIndex(self.index)
       self.direction = -1
       if self.command:
-        self.command(event)
         if application.remote_control_head:
           application.Hardware.RemoteCtlSend(f'{self.idName};{self.GetIndex()}\n')
+        self.command(event)
   def OnLeftDclick(self, event):	# Left double-click: Set index zero
     if not self.is_radio or self.GetValue():
       self.index = 0
       self.SetIndex(self.index)
       self.direction = 1
       if self.command:
-        self.command(event)
         if application.remote_control_head:
           application.Hardware.RemoteCtlSend(f'{self.idName};{self.GetIndex()}\n')
+        self.command(event)
   def DrawLabel(self, dc, width, height, dx=0, dy=0):
     QuiskCheckbutton.DrawLabel(self, dc, width, height, dx, dy)
     self.DrawGlyphCycle(dc, width, height)
@@ -1527,7 +1528,6 @@ class QuiskMenu(wx.Menu):
     if event:
       nid = event.GetId()
     item_text, handler = self.id2data[nid]
-    handler(event)
     if application.remote_control_head:
       menu_item = self.FindItemById(nid)
       if menu_item.IsCheckable():
@@ -1535,3 +1535,4 @@ class QuiskMenu(wx.Menu):
       else:
         checked = 0
       application.Hardware.RemoteCtlSend('MENU;%s;%s;%d\n' % (self.menu_name, item_text, int(checked)))
+    handler(event)
